==> download.php?file=Net2Plan-0.2.3.zip: com/net2plan/cli/CLINet2Plan.java <==
/*******************************************************************************
 * Copyright (c) 2013-2014 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.cli;

import com.jom.JOMException;
import com.net2plan.interfaces.networkDesign.Net2PlanException;
import com.net2plan.internal.Version;
import com.net2plan.utils.StringUtils;
import com.net2plan.internal.ICLIModule;
import com.net2plan.internal.CommandLineParser;
import com.net2plan.internal.ErrorHandling;
import com.net2plan.internal.SystemUtils;
import com.net2plan.internal.sim.SimKernel;
import com.net2plan.internal.sim.impl.ConnectionSimulation;
import com.net2plan.internal.sim.impl.ResilienceSimulation;
import com.net2plan.internal.sim.impl.TimeVaryingTrafficSimulation;
import com.net2plan.cli.tools.CLINetworkDesign;
import com.net2plan.cli.tools.CLIReport;
import com.net2plan.cli.tools.CLITrafficDesign;
import com.net2plan.utils.Triple;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.*;
import org.apache.commons.cli.*;

/**
 * Main class for the command-line user interface (CLI).
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.2.0
 */
public class CLINet2Plan
{
	private final static Map<String, Class<? extends ICLIModule>> modes;
	private final static Options options;
	private final static int lineWidth;

	static
	{
		lineWidth = 80;

		modes = new LinkedHashMap<String, Class<? extends ICLIModule>>();
		modes.put("net-design", CLINetworkDesign.class);
		modes.put("report", CLIReport.class);
		modes.put("traffic-design", CLITrafficDesign.class);
		modes.put("resilience-sim", ResilienceSimulation.class);
		modes.put("cac-sim", ConnectionSimulation.class);
		modes.put("time-varying-traffic-sim", TimeVaryingTrafficSimulation.class);

		options = new Options();

		Option help = OptionBuilder.withLongOpt("help")
				.withDescription("Show the complete help information. 'modename' is optional")
				.hasArg()
				.hasOptionalArg()
				.withArgName("modename")
				.create();

		Option mode = OptionBuilder.withLongOpt("mode")
				.withDescription("Mode: " + StringUtils.join(StringUtils.toArray(modes.keySet()), ", "))
				.hasArg()
				.withArgName("modename")
				.create();

		OptionGroup group = new OptionGroup();
		group.addOption(help);
		group.addOption(mode);

		options.addOptionGroup(group);
	}

	/**
	 * Default constructor.
	 * 
	 * @param args Command-line arguments
	 * @since 0.2.0
	 */
	public CLINet2Plan(String args[])
	{
		SystemUtils.configureEnvironment(CLINet2Plan.class, SystemUtils.UserInterface.CLI);

		System.out.println();

		CommandLineParser parser = new CommandLineParser();

		try
		{
			CommandLine cli = parser.parse(options, args);

			if (cli.hasOption("help"))
			{
				String mode = cli.getOptionValue("help");
				if (mode == null)
				{
					System.out.println(getCompleteHelp());
				}
				else
				{
					System.out.println(getModeHelp(mode));
				}
			}
			else if (!cli.hasOption("mode"))
			{
				System.out.println(getMainHelp());
			}
			else
			{
				String mode = cli.getOptionValue("mode");

				if (modes.containsKey(mode))
				{
					ICLIModule modeInstance = modes.get(mode).newInstance();

					try
					{
						modeInstance.executeFromCommandLine(args);
					}
					catch (Net2PlanException | JOMException ex)
					{
						System.out.println("Execution stopped");
						System.out.println();
						System.out.println(ex.getMessage());
					}
					catch (ParseException ex)
					{
						System.out.println("Bad syntax: " + ex.getMessage());
						System.out.println();
						System.out.println(getModeHelp(mode));
					}
					catch (Throwable ex)
					{
						Throwable ex1 = ErrorHandling.getInternalThrowable(ex);
						if (ex1 instanceof Net2PlanException || ex1 instanceof JOMException)
						{
							System.out.println("Execution stopped");
							System.out.println();
							System.out.println(ex1.getMessage());
						}
						else if (ex1 instanceof ParseException)
						{
							System.out.println("Bad syntax: " + ex1.getMessage());
							System.out.println();
							System.out.println(getModeHelp(mode));
						}
						else
						{
							System.out.println("Execution stopped. An unexpected error happened");
							System.out.println();
							printStackTrace(ex1);
						}
					}
				}
				else
				{
					throw new IllegalModeException("Bad mode - " + mode);
				}
			}
		}
		catch (IllegalModeException e)
		{
			System.out.println(e.getMessage());
			System.out.println();
			System.out.println(getMainHelp());
		}
		catch (ParseException e)
		{
			System.out.println("Bad syntax: " + e.getMessage());
			System.out.println();
			System.out.println(getMainHelp());
		}
		catch (Throwable e)
		{
			printStackTrace(e);
		}
	}

	private static void printStackTrace(Throwable throwable)
	{
		System.out.println(throwable);
		System.out.println();

		StackTraceElement[] stack = throwable.getStackTrace();
		for (StackTraceElement line : stack)
		{
			System.out.println(line);
		}
	}

	private static String getModeHelp(String mode)
	{
		StringBuilder help = new StringBuilder();

		if (!modes.containsKey(mode))
		{
			throw new IllegalModeException("Bad mode - " + mode);
		}

		try
		{
			ICLIModule modeInstance = modes.get(mode).newInstance();
			Options modeOptions = modeInstance.getCommandLineOptions();
			String modeHelp = modeInstance.getCommandLineHelp();

			StringWriter sw = new StringWriter();
			PrintWriter w = new PrintWriter(sw);
			HelpFormatter formatter = new HelpFormatter();
			formatter.printWrapped(w, lineWidth, "Mode: " + mode);
			formatter.printWrapped(w, lineWidth, "");
			formatter.printWrapped(w, lineWidth, modeHelp);
			formatter.printWrapped(w, lineWidth, "");
			formatter.printHelp(w, lineWidth, "java -jar Net2Plan-cli.jar --mode " + mode, null, modeOptions, 0, 1, null, true);

			if (modeInstance instanceof SimKernel)
			{
				formatter.printWrapped(w, lineWidth, "");
				formatter.printWrapped(w, lineWidth, "Simulation parameters:");
				formatter.printWrapped(w, lineWidth, "");

				List<Triple<String, String, String>> simParam = ((SimKernel) modeInstance).getSimulationParameters();
				for (Triple<String, String, String> param : simParam)
				{
					formatter.printWrapped(w, lineWidth, 2, String.format("- %s: %s. Default: %s", param.getFirst(), param.getThird(), param.getSecond()));
				}
			}
			w.flush();

			help.append(sw.toString());
		}
		catch (Throwable e)
		{
			throw new RuntimeException(e);
		}

		return help.toString();
	}

	private static String getMainHelp()
	{
		final StringBuilder help = new StringBuilder();

		HelpFormatter formatter = new HelpFormatter();
		StringWriter sw = new StringWriter();
		PrintWriter w = new PrintWriter(sw);
		formatter.printWrapped(w, lineWidth, new Version().toString() + " Command-Line Interface");
		formatter.printWrapped(w, lineWidth, "");
		formatter.printHelp(w, lineWidth, "java -jar Net2Plan-cli.jar", null, options, 0, 1, null, true);
		formatter.printWrapped(w, lineWidth, "");
		formatter.printWrapped(w, lineWidth, "Select 'help' to show this information, or 'mode' to execute a specific tool. Optionally, if 'help' is accompanied of a mode name, the help information for this mode is shown");
		w.flush();
		help.append(sw.toString());

		return help.toString();
	}

	private static String getCompleteHelp()
	{
		final StringBuilder help = new StringBuilder();
		final String lineSeparator = StringUtils.getLineSeparator();

		help.append(getMainHelp());

		for (String mode : modes.keySet())
		{
			help.append(lineSeparator);
			help.append(lineSeparator);
			help.append(getModeHelp(mode));
		}

		return help.toString();
	}

	/**
	 * Entry point for the command-line interface
	 * 
	 * @param args Command-line arguments
	 * @since 0.2.0
	 */
	public static void main(String[] args)
	{
		new CLINet2Plan(args);
	}

	private static class IllegalModeException extends RuntimeException
	{
		public IllegalModeException(String message)
		{
			super(message);
		}
	}
}

==> download.php?file=Net2Plan-0.3.0.zip: com/net2plan/cli/CLINet2Plan.java <==
/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.cli;

import com.jom.JOMException;
import com.net2plan.cli.tools.CLINetworkDesign;
import com.net2plan.cli.tools.CLIReport;
import com.net2plan.cli.tools.CLITrafficDesign;
import com.net2plan.interfaces.networkDesign.Net2PlanException;
import com.net2plan.internal.Version;
import com.net2plan.utils.StringUtils;
import com.net2plan.internal.plugins.ICLIModule;
import com.net2plan.internal.CommandLineParser;
import com.net2plan.internal.ErrorHandling;
import com.net2plan.internal.SystemUtils;
import com.net2plan.internal.sim.SimKernel;
import com.net2plan.internal.Constants.UserInterface;
import com.net2plan.internal.plugins.Plugin;
import com.net2plan.internal.plugins.PluginSystem;
import com.net2plan.internal.sim.OnlineSimulation;
import com.net2plan.utils.Triple;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.*;
import org.apache.commons.cli.*;

/**
 * Main class for the command-line user interface (CLI).
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.3.0
 */
public class CLINet2Plan
{
	private final static int lineWidth = 80;
	private final Map<String, Class<? extends ICLIModule>> modes;
	private final Options options;
	
	static
	{
		/* Add default CLI modules */
		PluginSystem.addPlugin(ICLIModule.class, CLINetworkDesign.class);
		PluginSystem.addPlugin(ICLIModule.class, CLIReport.class);
		PluginSystem.addPlugin(ICLIModule.class, CLITrafficDesign.class);
		PluginSystem.addPlugin(ICLIModule.class, OnlineSimulation.class);
	}

	/**
	 * Default constructor.
	 * 
	 * @param args Command-line arguments
	 * @since 0.2.0
	 */
	public CLINet2Plan(String args[])
	{
		SystemUtils.configureEnvironment(CLINet2Plan.class, UserInterface.CLI);
		System.out.println();
		modes = new LinkedHashMap<String, Class<? extends ICLIModule>>();
		options = new Options();
		
		try
		{
			for (Class<? extends Plugin> plugin : PluginSystem.getPlugins(ICLIModule.class))
			{
				ICLIModule instance = ((Class<? extends ICLIModule>) plugin).newInstance();
				modes.put(instance.getModeName(), instance.getClass());
			}

			Option helpOption = new Option("help", true, "Show the complete help information. 'modeName' is optional");
			helpOption.setArgName("modeName");
			helpOption.setOptionalArg(true);

			Option modeOption = new Option("mode", true, "Mode: " + StringUtils.join(StringUtils.toArray(modes.keySet()), ", "));
			modeOption.setArgName("modeName");
			modeOption.setOptionalArg(true);

			OptionGroup group = new OptionGroup();
			group.addOption(modeOption);
			group.addOption(helpOption);

			options.addOptionGroup(group);
			
			CommandLineParser parser = new CommandLineParser();
			CommandLine cli = parser.parse(options, args);

			if (cli.hasOption("help"))
			{
				String mode = cli.getOptionValue("help");
				System.out.println(mode == null ? getCompleteHelp() : getModeHelp(mode));
			}
			else if (!cli.hasOption("mode"))
			{
				System.out.println(getMainHelp());
			}
			else
			{
				String mode = cli.getOptionValue("mode");

				if (modes.containsKey(mode))
				{
					ICLIModule modeInstance = modes.get(mode).newInstance();

					try
					{
						modeInstance.executeFromCommandLine(args);
					}
					catch (Net2PlanException | JOMException ex)
					{
						if (ErrorHandling.isDebugEnabled()) ErrorHandling.printStackTrace(ex);
						
						System.out.println("Execution stopped");
						System.out.println();
						System.out.println(ex.getMessage());
					}
					catch (ParseException ex)
					{
						System.out.println("Bad syntax: " + ex.getMessage());
						System.out.println();
						System.out.println(getModeHelp(mode));
					}
					catch (Throwable ex)
					{
						Throwable ex1 = ErrorHandling.getInternalThrowable(ex);
						if (ex1 instanceof Net2PlanException || ex1 instanceof JOMException)
						{
							if (ErrorHandling.isDebugEnabled()) ErrorHandling.printStackTrace(ex);
							
							System.out.println("Execution stopped");
							System.out.println();
							System.out.println(ex1.getMessage());
						}
						else if (ex1 instanceof ParseException)
						{
							System.out.println("Bad syntax: " + ex1.getMessage());
							System.out.println();
							System.out.println(getModeHelp(mode));
						}
						else
						{
							System.out.println("Execution stopped. An unexpected error happened");
							System.out.println();
							ErrorHandling.printStackTrace(ex1);
						}
					}
				}
				else
				{
					throw new IllegalModeException("Bad mode - " + mode);
				}
			}
		}
		catch (IllegalModeException e)
		{
			System.out.println(e.getMessage());
			System.out.println();
			System.out.println(getMainHelp());
		}
		catch (ParseException e)
		{
			System.out.println("Bad syntax: " + e.getMessage());
			System.out.println();
			System.out.println(getMainHelp());
		}
		catch (InstantiationException | IllegalAccessException e)
		{
			ErrorHandling.printStackTrace(e);
		}
	}

	private String getModeHelp(String mode)
	{
		if (!modes.containsKey(mode)) throw new IllegalModeException("Bad mode - " + mode);

		try
		{
			ICLIModule modeInstance = modes.get(mode).newInstance();
			Options modeOptions = modeInstance.getCommandLineOptions();
			String modeHelp = modeInstance.getCommandLineHelp();

			StringWriter sw = new StringWriter();
			try (PrintWriter w = new PrintWriter(sw))
			{
				HelpFormatter formatter = new HelpFormatter();
				formatter.printWrapped(w, lineWidth, "Mode: " + mode);
				formatter.printWrapped(w, lineWidth, "");
				formatter.printWrapped(w, lineWidth, modeHelp);
				formatter.printWrapped(w, lineWidth, "");
				formatter.printHelp(w, lineWidth, "java -jar Net2Plan-cli.jar --mode " + mode, null, modeOptions, 0, 1, null, true);
				
				if (modeInstance instanceof SimKernel)
				{
					formatter.printWrapped(w, lineWidth, "");
					formatter.printWrapped(w, lineWidth, "Simulation parameters:");
					formatter.printWrapped(w, lineWidth, "");
					
					List<Triple<String, String, String>> simParam = ((SimKernel) modeInstance).getSimulationParameters();
					for (Triple<String, String, String> param : simParam)
					{
						formatter.printWrapped(w, lineWidth, 2, String.format("- %s: %s. Default: %s", param.getFirst(), param.getThird(), param.getSecond()));
					}
				}
				
				w.flush();
			}

			return(sw.toString());
		}
		catch (InstantiationException | IllegalAccessException e)
		{
			throw new RuntimeException(e);
		}
	}

	private String getMainHelp()
	{
		final StringBuilder help = new StringBuilder();

		HelpFormatter formatter = new HelpFormatter();
		StringWriter sw = new StringWriter();
		PrintWriter w = new PrintWriter(sw);
		formatter.printWrapped(w, lineWidth, new Version().toString() + " Command-Line Interface");
		formatter.printWrapped(w, lineWidth, "");
		formatter.printHelp(w, lineWidth, "java -jar Net2Plan-cli.jar", null, options, 0, 1, null, true);
		formatter.printWrapped(w, lineWidth, "");
		formatter.printWrapped(w, lineWidth, "Select 'help' to show this information, or 'mode' to execute a specific tool. Optionally, if 'help' is accompanied of a mode name, the help information for this mode is shown");
		w.flush();
		help.append(sw.toString());

		return help.toString();
	}

	private String getCompleteHelp()
	{
		final StringBuilder help = new StringBuilder();
		final String lineSeparator = StringUtils.getLineSeparator();

		help.append(getMainHelp());

		for (String mode : modes.keySet())
		{
			help.append(lineSeparator);
			help.append(lineSeparator);
			help.append(getModeHelp(mode));
		}

		return help.toString();
	}

	/**
	 * Entry point for the command-line interface.
	 * 
	 * @param args Command-line arguments
	 * @since 0.2.0
	 */
	public static void main(String[] args)
	{
		new CLINet2Plan(args);
	}

	private static class IllegalModeException extends RuntimeException
	{
		public IllegalModeException(String message)
		{
			super(message);
		}
	}
}
